/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.dev;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Evgeny Mandrikov
 * @goal check-dependencies
 * @requiresDependencyResolution runtime
 * @phase initialize
 */
public class CheckDependenciesMojo extends AbstractSonarPluginMojo {

  private static final String[] GWT_ARTIFACT_IDS = {"gwt-user", "gwt-dev", "sonar-gwt-api"};

  public void execute() throws MojoExecutionException, MojoFailureException {
    checkApiDependency();
    checkSonarDependencies();
    checkGwtDependencies();
    logCompileDependencies();
    logProvidedDependencies();
    logHelp();
  }

  private void checkSonarDependencies() throws MojoExecutionException {
    List<String> ids = new ArrayList<String>();
    for (Artifact dep : getDependencyArtifacts(Artifact.SCOPE_COMPILE)) {
      if ("org.codehaus.sonar".equals(dep.getGroupId())) {
        ids.add(dep.getDependencyConflictId());
      }
    }
    if (!ids.isEmpty()) {
      throw new MojoExecutionException("The dependencies on Sonar libraries should be declared with scope 'provided': "
          + StringUtils.join(ids, ", "));
    }
  }

  private void logHelp() {
    StringBuilder message = new StringBuilder();
    message.append('\n')
        .append("\nSee following page for more details about plugin dependencies:")
        .append("\n\thttp://docs.codehaus.org/display/SONAR/Developing+a+plugin#Developingaplugin-Dependencymanagement")
        .append('\n');
    getLog().info(message.toString());
  }

  private void checkGwtDependencies() {
    List<String> ids = new ArrayList<String>();
    for (Artifact dep : getDependencyArtifacts(Artifact.SCOPE_COMPILE)) {
      if (ArrayUtils.contains(GWT_ARTIFACT_IDS, dep.getArtifactId())) {
        ids.add(dep.getDependencyConflictId());
      }
    }
    if (!ids.isEmpty()) {
      getLog().warn(getMessage("Following GWT dependencies should be defined with scope 'provided':", ids));
    }
  }

  private void logCompileDependencies() {
    List<String> ids = new ArrayList<String>();
    for (Artifact dep : getDependencyArtifacts(Artifact.SCOPE_COMPILE)) {
      ids.add(dep.getDependencyConflictId());
    }
    if (!ids.isEmpty()) {
      getLog().info(getMessage("Following dependencies are packaged in the plugin:", ids));
    }
  }

  private void logProvidedDependencies() {
    List<String> ids = new ArrayList<String>();
    for (Artifact dep : getDependencyArtifacts(Artifact.SCOPE_PROVIDED)) {
      ids.add(dep.getDependencyConflictId());
    }
    if (!ids.isEmpty()) {
      getLog().debug(getMessage("Ignored dependencies:", ids));
    }
  }

  private String getMessage(String title, List<String> ids) {
    StringBuilder message = new StringBuilder();
    message.append(title);
    message.append("\n");
    for (String id : ids) {
      message.append("\t").append(id).append("\n");
    }
    return message.toString();
  }

  private void checkApiDependency() throws MojoExecutionException {
    Artifact sonarApi = getSonarPluginApiArtifact();

    if (sonarApi == null) {
      throw new MojoExecutionException(
          SONAR_PLUGIN_API_GROUPID + ":" + SONAR_PLUGIN_API_ARTIFACTID + " should be declared in dependencies"
      );
    }

    if (!Artifact.SCOPE_PROVIDED.equals(sonarApi.getScope())) {
      throw new MojoExecutionException(
          SONAR_PLUGIN_API_GROUPID + ":" + SONAR_PLUGIN_API_ARTIFACTID + " should be declared with scope 'provided'"
      );
    }
  }
}
