/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.dev;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.archiver.MavenArchiveConfiguration;
import org.apache.maven.archiver.MavenArchiver;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.resolver.filter.ScopeArtifactFilter;
import org.apache.maven.model.License;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.codehaus.plexus.archiver.jar.JarArchiver;
import org.codehaus.plexus.util.FileUtils;
import org.sonar.pluginmetadata.PluginManifest;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Build a Sonar Plugin from the current project.
 *
 * @author Evgeny Mandrikov
 * @goal sonar-plugin
 * @phase package
 * @requiresProject
 * @requiresDependencyResolution runtime
 */
public class SonarPluginMojo extends AbstractSonarPluginMojo {
  private static final String LIB_DIR = "META-INF/lib/";

  /**
   * The Jar archiver.
   *
   * @component role="org.codehaus.plexus.archiver.Archiver" role-hint="jar"
   */
  protected JarArchiver jarArchiver;

  /**
   * The archive configuration to use.
   * See <a href="http://maven.apache.org/shared/maven-archiver/index.html">Maven Archiver Reference</a>.
   *
   * @parameter
   */
  private MavenArchiveConfiguration archive = new MavenArchiveConfiguration();

  public void execute() throws MojoExecutionException, MojoFailureException {
    File jarFile = createArchive();
    String classifier = getClassifier();
    if (classifier != null) {
      projectHelper.attachArtifact(getProject(), "jar", classifier, jarFile);
    } else {
      getProject().getArtifact().setFile(jarFile);
    }
  }

  public File createArchive() throws MojoExecutionException {
    checkPluginClass();

    File jarFile = getJarFile(getOutputDirectory(), getFinalName(), getClassifier());
    MavenArchiver archiver = new MavenArchiver();
    archiver.setArchiver(jarArchiver);
    archiver.setOutputFile(jarFile);
    try {
      archiver.getArchiver().addDirectory(getClassesDirectory());

      archive.addManifestEntry(PluginManifest.KEY, getPluginKey());
      archive.addManifestEntry(PluginManifest.NAME, getPluginName());
      archive.addManifestEntry(PluginManifest.DESCRIPTION, getPluginDescription());
      archive.addManifestEntry(PluginManifest.VERSION, getProject().getVersion());
      archive.addManifestEntry(PluginManifest.MAIN_CLASS, getPluginClass());
      archive.addManifestEntry(PluginManifest.HOMEPAGE, getPluginUrl());
      archive.addManifestEntry(PluginManifest.SONAR_VERSION, getSonarPluginApiArtifact().getVersion());
      archive.addManifestEntry(PluginManifest.LICENSE, getPluginLicense());
      archive.addManifestEntry(PluginManifest.ORGANIZATION, getPluginOrganization());
      archive.addManifestEntry(PluginManifest.ORGANIZATION_URL, getPluginOrganizationUrl());
      archive.addManifestEntry(PluginManifest.CATEGORY, getPluginCategory());

      List<String> libs = processDependencies();
      if (!libs.isEmpty()) {
        archiver.getArchiver().addDirectory(getAppDirectory());
        archive.addManifestEntry(PluginManifest.DEPENDENCIES, StringUtils.join(libs, " "));
      }

      archiver.createArchive(getProject(), archive);

      return jarFile;
    } catch (Exception e) {
      throw new MojoExecutionException("Error assembling Sonar-plugin: " + e.getMessage(), e);
    }
  }

  private String getPluginLicense() {
    List<String> licenses = new ArrayList<String>();
    if (getProject().getLicenses() != null) {
      for (Object license : getProject().getLicenses()) {
        License l = (License) license;
        if (l.getName() != null) {
          licenses.add(l.getName());
        }
      }
    }
    return StringUtils.join(licenses, " ");
  }

  private String getPluginOrganization() {
    if (getProject().getOrganization() != null) {
      return getProject().getOrganization().getName();
    }
    return null;
  }

  private String getPluginOrganizationUrl() {
    if (getProject().getOrganization() != null) {
      return getProject().getOrganization().getUrl();
    }
    return null;
  }


  private void checkPluginClass() throws MojoExecutionException {
    if (!new File(getClassesDirectory(), getPluginClass().replace('.', '/') + ".class").exists()) {
      throw new MojoExecutionException("Error assembling Sonar-plugin: Plugin-Class '" + getPluginClass() + "' not found");
    }
  }

  protected static File getJarFile(File basedir, String finalName, String classifier) {
    if (classifier == null) {
      classifier = "";
    } else if (classifier.trim().length() > 0 && !classifier.startsWith("-")) {
      classifier = "-" + classifier;
    }
    return new File(basedir, finalName + classifier + ".jar");
  }

  private List<String> processDependencies() throws IOException {
    List<String> libs = new ArrayList<String>();
    File libDirectory = new File(getAppDirectory(), LIB_DIR);
    Set<Artifact> artifacts = getArtifacts();
    ScopeArtifactFilter filter = new ScopeArtifactFilter(Artifact.SCOPE_RUNTIME);
    for (Artifact artifact : artifacts) {
      String targetFileName = getDefaultFinalName(artifact);
      if (filter.include(artifact)) {
        FileUtils.copyFileIfModified(artifact.getFile(), new File(libDirectory, targetFileName));
        getLog().debug("Add library to plugin dependencies: " + targetFileName);
        libs.add(LIB_DIR + targetFileName);
      }
    }
    return libs;
  }

  private String getDefaultFinalName(Artifact artifact) {
    return artifact.getArtifactId() + "-" + artifact.getVersion() + "." + artifact.getArtifactHandler().getExtension();
  }

  @SuppressWarnings({"unchecked"})
  protected Set<Artifact> getArtifacts() {
    return getProject().getArtifacts();
  }
}
